/*http://math.furman.edu/~dcs/java/  */
import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.sun.j3d.utils.universe;

public class Pendulum extends Applet implements MouseListener {
    int x_win_int = 500, y_win_int = 430, x0_int, y0_int;
    int method = 1;
    double x_win, y_win, x_scale, y_scale, x0, y0;
    double x_max = 10., y_max = 4., dt = 0.005;

    public void init() {
	CheckboxGroup mcbg = new CheckboxGroup();
	Checkbox cb1 = new Checkbox("Euler",true, mcbg);
	Checkbox cb2 = new Checkbox("Runge-Kutta 2",false, mcbg);
	Checkbox cb3 = new Checkbox("Runge-Kutta 4",false, mcbg);
	cb1.addItemListener(new ItemListener() {
	    public void itemStateChanged(ItemEvent ie) {
		method = 1;
	    }
	});
	cb2.addItemListener(new ItemListener() {
	    public void itemStateChanged(ItemEvent ie) {
		method = 2;
	    }
	});
	cb3.addItemListener(new ItemListener() {
	    public void itemStateChanged(ItemEvent ie) {
		method = 3;
	    }
	});
	this.add(cb1);
	this.add(cb2);
	this.add(cb3);
	Button continue_button = new Button("Continue");
	continue_button.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		draw();
		}
	});
	this.add(continue_button);
	Button clear_button = new Button("Clear");
	clear_button.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		initial_conditions();
		repaint();
	    }
	});
	this.add(clear_button);
	this.addMouseListener(this);
       	this.setBackground(Color.white);
	x_win = ((double) x_win_int)/2.;
	y_win = ((double) y_win_int)/2.;
	x_scale = x_win/x_max;
	y_scale = y_win/y_max;
	initial_conditions();
    }

    public void initial_conditions() {
	x0 = 0;
	y0 = 0;
	x0_int = (int) (x_scale*x0 + x_win);
	y0_int = (int) (y_scale*y0 + y_win + 15);
    }

    public void paint(Graphics g) {
	int i;
	g.setColor(Color.white);
	g.fillRect(0,0,x_win_int,y_win_int);
	g.setColor(Color.blue);
	g.fillRect(0,0,x_win_int,30);
	g.drawRect(0,0,x_win_int-1,y_win_int-1);
	g.setColor(Color.black);
	g.drawLine(0,229,499,229);
	g.drawLine(249,30,249,429);
	for(i= (int)-x_max+1;i<(int) x_max;i++){
	    g.drawLine(249+i*(int)x_scale,225,249+i*(int)x_scale,233);
	}
	for(i= (int)-y_max+1;i<(int) y_max;i++){
	    g.drawLine(245,229+i*(int)y_scale,253,229+i*(int)y_scale);
	}

    }

    public void  mouseClicked(MouseEvent e) {
	x0_int = e.getX();
	y0_int = e.getY();
	x0 = (double) x0_int;
	y0 = (double) y0_int;
	x0 = (x0 - x_win) / x_scale;
	y0 = (y_win + 15. - y0) / y_scale;
	draw();
    }

    public void draw() {
	int i,x_int, y_int;
	double x, y;
	Graphics g = this.getGraphics();
	switch (method) {
	case 1:
	    g.setColor(Color.red);
	    break;
	case 2:
	    g.setColor(Color.green);
	    break;
	case 3:
	    g.setColor(Color.blue);
	}
	for(i=1;i<=1000;i++) {
	    switch (method) {
	    case 1:
		x = euler_x(x0,y0);
		y = euler_y(x0,y0);
		break;
	    case 2:
		x = rk2_x(x0,y0);
		y = rk2_y(x0,y0);
		break;
	    default:
		x = rk4_x(x0,y0);
		y = rk4_y(x0,y0);
	    }
	    x_int = (int) (x_scale*x + x_win);
	    y_int = (int) (y_win + 15. - y_scale*y);
	    if ((y_int > 30) && (y0_int > 30))
		g.drawLine(x0_int, y0_int, x_int, y_int);
	    x0 = x;
	    y0 = y;
	    x0_int = x_int;
	    y0_int = y_int;
	}
    }

    public double f_x(double x, double y) {
	return y;
    }

    public double f_y(double x, double y) {
	return -Math.sin(x);
    }

    public double euler_x(double x, double y) {
	return x + f_x(x,y)*dt;
    }

    public double euler_y(double x, double y) {
	return y + f_y(x,y)*dt;
    }

    public double rk2_x(double x, double y) {
	return x + f_x(x + f_x(x,y)*dt/2.,y + f_y(x,y)*dt/2.)*dt;
    }

    public double rk2_y(double x, double y) {
	return y + f_y(x + f_x(x,y)*dt/2.,y + f_y(x,y)*dt/2.)*dt;
    }

    public double rk4_x(double x, double y) {
	double mx1, mx2, mx3, mx4, my1, my2, my3;
	mx1 = f_x(x,y)*dt;
	my1 = f_y(x,y)*dt;
	mx2 = f_x(x + mx1/2., y + my1/2.)*dt;
	my2 = f_y(x + mx1/2., y + my1/2.)*dt;
	mx3 = f_x(x + mx2/2., y + my2/2.)*dt;
	my3 = f_y(x + mx2/2., y + my2/2.)*dt;
	mx4 = f_x(x + mx3, y + my3)*dt;
	return x + mx1/6. + mx2/3. + mx3/3. + mx4/6.;
    }

    public double rk4_y(double x, double y) {
	double mx1, mx2, mx3, my1, my2, my3, my4;
	mx1 = f_x(x,y)*dt;
	my1 = f_y(x,y)*dt;
	mx2 = f_x(x + mx1/2., y + my1/2.)*dt;
	my2 = f_y(x + mx1/2., y + my1/2.)*dt;
	mx3 = f_x(x + mx2/2., y + my2/2.)*dt;
	my3 = f_y(x + mx2/2., y + my2/2.)*dt;
	my4 = f_y(x + mx3, y + my3)*dt;
	return y + my1/6. + my2/3. + my3/3. + my4/6.;
    }

    public void mouseReleased(MouseEvent e) {;}
    public void mousePressed(MouseEvent e) {;}
    public void mouseEntered(MouseEvent e) {;}
    public void mouseExited(MouseEvent e) {;}


public void adds(Graphics2D g3d)
{
	;
}
}